
/* udpcons.c -- an UDP console for linux-2.4
 *
 * Copyright (c) 2001 Alessandro Rubini <rubini@gnu.org>
 *
 * This program is Free Software, licensed according to the GNU GPL
 */

#ifndef __KERNEL__
#  define __KERNEL__
#endif

#ifndef MODULE
#  define MODULE
#endif

#include <linux/modversions.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/console.h>
#include <linux/net.h>
#include <linux/skbuff.h>
#include <linux/inet.h>
#include <linux/ip.h>
#include <linux/udp.h>
#include <linux/init.h>

MODULE_AUTHOR("Alessandro Rubini");


/* defaults */
#define UDPCONS_DEFAULT_PORT 2222
#define UDPCONS_DEFAULT_DEST "255.255.255.255"


/* parameters */
int port = 0;
char *dest = NULL;

MODULE_PARM(port, "i");
MODULE_PARM(dest, "s");


/* prefixed versions, for better readability of the code */
int udpcons_port;
char *udpcons_dest;

__u32 udpcons_destaddr;

/* setup function */
int udpcons_setup(struct console *co, char *options)
{
    /*
     * This is only called if a kernel command line is passed.
     * The order of precedence is: module parameter, cmdline, default.
     * Thus, values are only set here if still undefined; defaults
     * are set by udpcons_init if neither a parameter nor a cmdline argument
     * has been specified
     */
    if (co->index > 0 && !udpcons_port) 
	udpcons_port = co->index;

    if (options && !udpcons_dest)
	udpcons_dest = options;

    return 0; /* success */
}


int udpcons_waitkey(struct console *co)
{
    /*
     * well, to implement this one in a sensible way, the module
     * should wait for a packet sent to its port. But to do this
     * you need a bound socket, so I'd better not do that in this
     * simple module
     */
    return '\n';
}

void udpcons_write(struct console *co, const char *s, unsigned count)
{
    struct sk_buff *skb;

    skb = dev_alloc_skb(count + sizeof(struct iphdr) + sizeof(struct udphdr));
    

    ip_queue_xmit(skb); /* net/ipv4/ip_output.c */
    dev_queue_xmit(skb); /* net/core/dev.c */
}




struct console udpcons = {
    name:     "udp",
    write:    udpcons_write,
    wait_key: udpcons_waitkey,
    setup:    udpcons_setup,
    device:   NULL, /* no tty device associated to UDP */
    flags:    CON_ENABLED, /* always enabled */
    index:    -1, /* unspecified */
};


int udpcons_init(void)
{
    /* copy simple-name module parameters to prefixed-name variables */
    udpcons_port = port;
    udpcons_dest = dest;

    /* register this console, if a cmdline argument matches, it calls setup */
    register_console(&udpcons);

    /* if still no values for port and dest, use defaults */
    if (!udpcons_port) udpcons_port = UDPCONS_DEFAULT_PORT;
    if (!udpcons_dest) udpcons_dest = UDPCONS_DEFAULT_DEST;
    
    /* convert the destination IP (no error check, though */
    udpcons_destaddr = in_aton(udpcons_dest);

    printk(KERN_INFO "udp console registered, port %i, IP \"%s\" (0x%08lx)\n",
	   udpcons_port, udpcons_dest, (unsigned long)udpcons_destaddr);

    return 0;
}

void udpcons_exit(void)
{
    printk(KERN_INFO "udp console being unregistered\n");
    unregister_console(&udpcons); /* return 0 for success */
    return;
}

module_init(udpcons_init);
module_exit(udpcons_exit);

